classdef AOROIctrl < handle

    properties
    hSI
    hSIctrl 
    hSHctrl
    hDMctrl
    hGUI
    hCameraGUI
    hAOdata%handels to other objects
    
    %internal properties
    wfroidata=[];%struct storing relavent rois
    mroidata=[]; %roigroup when mroi is used for imaging
    
    %deformable mirror data from WF calculations
    sampleDMdata;
    systemDMdata;
    applyDMdata;
    
    %ROIsize properties (to be assigned)
    umperdeg; %micrometer per degree, depends on optics, here set to a convenient value.
    cohlength; %maximum coherent length in tissue, 
    cohlengthZ ;
    maxdeg;
    
    
    %scanning contol properties
    isscanning=false; %whether SI is scanning
    useaonxtgrb;
    tousesampleWF;
    tousesystemWF;
    %wavefront measure control properties
    isrunningWF
    
    useZscan
    useDM=0; % indicate if need to load data to DM when Aq or Focus Start.
    ROIalongZ %creat auto ROI along Z or not.
    fromZ
    toZ % auto ROI z direction properties
    shrinkfraction
    ROIsize  %pixal size of ROI created from auto R
    
    end
    
    %lifecycle
    methods
        function obj=AOROIctrl(SI,SIctrl)
            obj.hSI=SI;
            obj.hSIctrl=SIctrl;
            obj.hSHctrl=AOcontrol.SHctrl(obj);    %creation of AO ctrl obj goes here
            obj.hDMctrl=AOcontrol.DMctrl();         %creation of DM control object.
            % default values
            obj.umperdeg=17;
            obj.cohlength=50; 
            obj.cohlengthZ=50;           
            obj.ROIalongZ=0; %creat auto ROI along Z or not.
            obj.fromZ=0;
            obj.toZ=0 ;
            obj.useZscan=0;

            obj.shrinkfraction=1;
            obj.ROIsize = [256,256];
            obj.useaonxtgrb=0;
            obj.maxdeg=obj.cohlength/obj.umperdeg;
           

            obj.hGUI=AOGUI(obj);
            % send defaults to GUI.
            GUIhandles=guidata(obj.hGUI);
            GUIhandles.umperdeg.String=num2str(obj.umperdeg);
            GUIhandles.Shrinkfraction.String=num2str(obj.shrinkfraction);
            GUIhandles.cohlength.String=num2str(obj.cohlength);
            GUIhandles.cohlengthZ.String=num2str(obj.cohlengthZ);
            GUIhandles.fromZ.String=num2str(obj.fromZ);
            GUIhandles.toZ.String=num2str(obj.toZ);
        end
        
        function delete(obj)
            if ishandle(obj.hGUI)
                close(obj.hGUI)
            end
        end
        
    end
    %user methods
    methods 
        
        
        
        function saveWFROI(obj)
            
           %check roi Group validity first
            roig=obj.hSI.hRoiManager.roiGroupMroi; %get current roi group stored by roi manager
           
            %obj.roidata=[1];
            %check if roi is rotated square, and only have one scanfield.
            rois=roig.rois;
            nrois=length(rois);
            if nrois==0
                msgbox('No roi in SI','Error')
                return
            end
                %fprintf('saving current ROIs')
               obj.wfroidata=copy(roig);
               %copy the roigroup, since the origional could be cleared.
               msgbox('Saved')
            
        end
        
        
        function saveMROI(obj)
            
           %check roi Group validity first
            roig=obj.hSI.hRoiManager.roiGroupMroi; %get current roi group stored by roi manager
           
            %obj.roidata=[1];
            %check if roi is rotated square, and only have one scanfield.
            rois=roig.rois;
            nrois=length(rois);
            if nrois==0
                msgbox('No roi in SI','Error')
                return
            end
                %fprintf('saving current ROIs')
               obj.mroidata=copy(roig);
               %copy the roigroup, since the origional could be cleared.
               msgbox('Saved')
            
            %after saving mroi, if wfroi is not empty, create list of mroi
            %in pop-up manue. Using criteria of whether center of WFroi is
            %in that mroi.
            if ~isempty(obj.wfroidata) %if wfroidata is loaded, thus it's important to create wfroidata first.
                nrois=length(obj.mroidata.rois);
                obj.hGUI.Children(obj.findchild(obj.hGUI.Children,'MROImanu')).String=[1:nrois]';
                
            end
            
        end
        
        function setWFROI(obj,indROI)
            % this function set the current mroi to rois in WFroi that is
            % contained by mroi(indROI)
            if ~obj.hSI.hRoiManager.mroiEnable==1
                msgbox('not in mroi imaging mode','error')
                return
                
            end
            currentscanfield=obj.mroidata.rois(indROI).scanfields;
            %for simplicity rotation not taken into account here (nobody need totation anyways~)
            minx=currentscanfield.centerXY(1)-currentscanfield.sizeXY(1)/2;
            maxx=currentscanfield.centerXY(1)+currentscanfield.sizeXY(1)/2;
            miny=currentscanfield.centerXY(2)-currentscanfield.sizeXY(2)/2;
            maxy=currentscanfield.centerXY(2)+currentscanfield.sizeXY(2)/2;
            
            wfrois=obj.wfroidata.rois;
            nWFroi=length(wfrois);
            wfroigrp=scanimage.mroi.RoiGroup();
            for i=1:nWFroi
               if minx<wfrois(i).scanfields.centerXY(1)&&wfrois(i).scanfields.centerXY(1)<maxx&&miny<wfrois(i).scanfields.centerXY(2)&&wfrois(i).scanfields.centerXY(2)<maxy
                   wfroigrp.add(copy(wfrois(i)));
               end
            end
            if isempty(wfroigrp.rois)
                msgbox('no WF ROI exist in this MROI','error')
                return
            else
            obj.hSI.hRoiManager.roiGroupMroi=copy(wfroigrp);
            msgbox('wf rois loaded in scanimage','success')
            end
            
        end
        
        function importWFROI(obj)
            if isempty(obj.wfroidata)
                msgbox('no ROI data saved','Error')
                return
            else
            %fprintf('importing ROIs to SI')
           
            obj.hSI.hRoiManager.roiGroupMroi=copy(obj.wfroidata);
            %copy back roi data
            %msgbox('ROI data imported to ScanImage')
            end
        end
        
        function importMROI(obj)
            if isempty(obj.mroidata)
                msgbox('no ROI data saved','Error')
                return
            else
            %fprintf('importing ROIs to SI')
           
            obj.hSI.hRoiManager.roiGroupMroi=copy(obj.mroidata);
            %copy back roi data
            %msgbox('ROI data imported to ScanImage')
            end
        end
        
        function callROImanager(obj)
            obj.hSIctrl.editImagingRoiGroup
            %fprintf('calling ROI manager')
        end
        
        function autoROI(obj)
            %log current scanning field
            scanfgrp=obj.hSI.hRoiManager.currentRoiGroup; %current ROI group
            obj.maxdeg=obj.cohlength/obj.umperdeg;
            if ~strcmp(scanfgrp.name,'Default Imaging ROI Group') %check if current ROI group is default
                msgbox('Not in field scanning mode, cannot auto ROI during ROI imaging','Error')
                return
            else
                if obj.ROIalongZ==0
                    nZ=0;
                    cellsizeZ=-obj.fromZ;
                else
                    nZ=ceil(abs(obj.fromZ-obj.toZ)/obj.cohlengthZ);
                    cellsizeZ=abs(obj.fromZ-obj.toZ)/nZ;
                end
                scanf=scanfgrp.rois.scanfields;%
                nXY=ceil(scanf.sizeXY/obj.maxdeg); %number of cells, keeping dimension smaller than cohlength
                cellsizeXY=scanf.sizeXY./nXY; %cell size
                %pixresXY=ceil(scanf.pixelResolutionXY./nXY);%pixal resolution for each ROI.
                pixresXY=obj.ROIsize;
                autoROIgrp=scanimage.mroi.RoiGroup();%empty ROI group to add auto ROIs
                
                
               for k=1:nZ+1
                
                for j=1:nXY(2) 
                    
                    
                   for i=1:nXY(1)
                       autoscanf(i,j,k)=scanimage.mroi.scanfield.fields.RotatedRectangle(); %create a scanfield 
                       autoscanf(i,j,k).pixelResolutionXY=pixresXY;
                       autoscanf(i,j,k).sizeXY=cellsizeXY;
                       autoscanf(i,j,k).pixelRatio=pixresXY./cellsizeXY;
                       
                       autoscanf(i,j,k).centerXY=scanf.centerXY-scanf.sizeXY/2+([i,j]-0.5).*cellsizeXY;
                       autoROI(i,j,k)=scanimage.mroi.Roi();
                       autoROI(i,j,k).add((obj.fromZ+k*cellsizeZ),autoscanf(i,j,k));
                       autoROIgrp.add(autoROI(i,j,k));
                   end
                end
               end
                
                obj.hSI.hRoiManager.roiGroupMroi=copy(autoROIgrp);
                msgbox('auto ROI grid created and imported to ScanImage')
            end
            
            
            %fprintf('autoROI created, logged to system')
            
        end
        
        function clearROIs(obj)
            obj.wfroidata=[];
            obj.mroidata=[];
            msgbox('ROI data cleared')
            %fprintf('clearROIs')
        end
        
       
        
        function getwavefront(obj)
            %zscan function not implemented yet, because not sure how fastZ scan
            %work.
            
            
            if isempty(obj.roidata)
                msgbox('No ROI data loaded','Error');
                return %abort wavefront sensing if no ROI data loaded
            end
            
            if obj.hSHctrl.iscameraready==0
                msgbox('Camera is not ready yet')
                return
            else
                obj.isrunningWF=true;
                obj.hAOdata=AOcontrol.AOdata(obj.roidata);
                obj.hAOdata.zscanused=obj.useZscan;
                obj.hAOdata.shrinkfraction=obj.shrinkfraction;
                
                assignin('base','hAOdata',obj.hAOdata) %creation of AOdata object in workspace
                % file name is uuid number of ROIGroup used
                uuid=obj.hAOdata.ROIGroup.uuid;
                filename=strcat('WFpics\',uuid,'.tif');
                obj.hAOdata.filename=filename;
                tif=Tiff(filename,'w');
                %fprintf('wavefrontsensing percedure')
                nROIs=length(obj.roidata.rois);
                obj.hAOdata.nROI=nROIs;
                % take background
                bgdata=obj.hSHctrl.takeSHdata();
            
                tif.setTag('Photometric',Tiff.Photometric.MinIsWhite);
                tif.setTag('Photometric',Tiff.Photometric.MinIsWhite);
                tif.setTag('Compression',Tiff.Compression.None);
                tif.setTag('ImageLength',obj.hSHctrl.YPixels)
                tif.setTag('ImageWidth',obj.hSHctrl.XPixels)
                tif.setTag('BitsPerSample',16)
                tif.setTag('SampleFormat',Tiff.SampleFormat.Int)
                tif.setTag('PlanarConfiguration',Tiff.PlanarConfiguration.Chunky);
                tif.write(bgdata);
                tif.writeDirectory;
                
                for i=1:nROIs
                   
                   currentROI=copy(obj.hAOdata.ROIGroup.rois(i));%load i th ROI from AOdata object.
                   %currentROI.scanfields.pixelResolutionXY=floor(currentROI.scanfields.pixelResolutionXY*obj.shrinkfraction);
                   %do not change pixal resolution during shrinking anymore.
                   
                   %modify ROI by shrink fraction
                   
                   %currentROI.scanfields.pixalResolutionXY(1)=floor(currentROI.scanfields.pixalResolutionXY*obj.shrinkfraction);
                   %currentROI.scanfields.sizeXY(1)=floor(currentROI.scanfields.sizeXY*obj.shrinkfraction);
                   
                   %use the code above if only want to shrink on x
                   %dimension.
                   obj.roidata.clear();
                   obj.roidata.add(currentROI);%only current ROI in roidata.
                   obj.roidata.rois(1).scanfields.sizeXY=obj.roidata.rois(1).scanfields.sizeXY*obj.shrinkfraction;
                   obj.roidata.rois(1).scanfields.sizeXY(1)=obj.roidata.rois(1).scanfields.sizeXY(1)*obj.hSI.hScan2D.fillFractionSpatial;
                   obj.importROIs;
                   obj.hSI.hRoiManager.mroiEnable=1;
                   obj.hSI.startFocus();%start scanning
                   SHdata=obj.hSHctrl.takeSHdata(); 
                   obj.hSI.abort();
                   %assign properties to tif file, write image in the order
                   %taken to tiff
                   tif.setTag('Photometric',Tiff.Photometric.MinIsWhite);
                   tif.setTag('Photometric',Tiff.Photometric.MinIsWhite);
                   tif.setTag('Compression',Tiff.Compression.None);
                   tif.setTag('ImageLength',obj.hSHctrl.YPixels)
                   tif.setTag('ImageWidth',obj.hSHctrl.XPixels)
                   tif.setTag('BitsPerSample',16)
                   tif.setTag('SampleFormat',Tiff.SampleFormat.Int)
                   tif.setTag('PlanarConfiguration',Tiff.PlanarConfiguration.Chunky);
                   tif.write(SHdata);
                   tif.writeDirectory;
                end
                delete(tif);
                msgbox('Done!')
                obj.roidata=obj.hAOdata.getROIGroup;
                obj.importROIs();
            end
              
        end
        
        function abortwavefrontmeasurment(obj)
%             if obj.isrunningWF==true
%                 fprintf('abort AOdata')
%                 
%                 obj.isrunningAOdata=false;
%             else
%                return
%             end
        msgbox('Press Ctrl + C !')
        end
        
        function calculateWF(obj)
            % calculate wavefront and DM data for current AOdata object.
            %
            if isempty(obj.hAOdata)
                msgbox('no Wave Front object in AO controller','Error')
                return
              
            end
            
            if exist(obj.hAOdata.filename,'file')==0
                msgbox('image file correspond to Wave Front object not found','Error')
                return
            end
            
            if ~isempty(obj.hAOdata.wavefrontdata)
                msgbox('overwriting existing wavefront data')
                
            end
            
            AOcontrol.calculateWF(obj.hAOdata);
            
            
                
            
            
        end
        
        
        function savetofile(obj)
            %save AOdata object to file
            hAOdata=obj.hAOdata;
            if ~isequal(class(hAOdata),'AOcontrol.AOdata')
               msgbox('no wavefront data detected','Error') 
               return
            end
            
            uisave('hAOdata','AOdata1');
            
        end
        
        function loadfile(obj)
            %loadfile will load .mat file that contain AOdata to AO control system, and change
            %the roi group stored to that correspond to that wavefront data.
            [filename,~,fi]=uigetfile('*.mat','Choose Wavefront data file');
            if fi==0
                return
            end
            load(filename);
            if ~exist('hAOdata','var')==1
                msgbox('not an AOdata file','Error')
                return
            end
             assignin('base','hAOdata',hAOdata);
             obj.roidata=hAOdata.ROIGroup;
             obj.hAOdata=hAOdata;
        end
        
        function ready=prepareaonxtgrb(obj) 
            %function that prepare system for ao on next grab
            %for segmented FOV AO scanning.
            %will load ROI to scanimage.
            %fprintf('preparing ao for next grab')
            %checking AOdata object exist.
            AOdata=obj.hAOdata;
            if ~isequal(class(AOdata),'AOcontrol.AOdata')
               msgbox('no wavefront data saved','Error')
               ready=0;
               return
            end
            
            if ~AOdata.iscompelete(obj.usesystemWF,obj.usesampleWF)
                msgbox('Wavefront data incompelete','Error')
                ready=0;
                return
            end
            
            obj.roidata=AOdata.getROIGroup;
            obj.hSI.hRoiManager.roiGroupMroi=copy(obj.roidata);
            obj.hSI.hRoiManager.mroiEnable=true;
            obj.useaonxtgrb=1;
            ready=1;
            msgbox('Ready, donnot change setting before starting')
            %still need to load DM data and timing information.
            
        end
        
        function disbleaonxtgrb(obj)
            obj.useaonxtgrb=0;
        end
        
        
        
        function AOAqstart(obj)
            %fprintf('AOAqstart')
            %called everytime, but only make difference when AO is enabled
            if obj.useaonxtgrb==1
                switch [obj.usesystemWF,obj.usesampleWF]
                    case [1,1]
                        obj.hDMctrl.sendpatterndata(obj.hAOdata.getcombinedDMdata);
                        obj.hDMctrl.triggeron();
                    case [0,1]
                        obj.hDMctrl.sendpatterndata(obj.hAOdata.getsampleDMdata);
                        obj.hDMctrl.triggeron();
                    case [1,0]
                        obj.hDMctrl.sendpatterndata(obj.hAOdata.getsystemDMdata);
                        obj.hDMctrl.triggeron();
                    case [0,0]
                        msgbox('no DM data selected, DM not used')
                end
            end
        end
        
        function AOAqend(obj)
            %fprintf('AOAqend')
            if obj.useaonxtgrb==1
            obj.hDMctrl.stopDM();
            obj.hDMctrl.triggeroff();
            obj.hDMctrl.resetDM();
            end
        end
        
        function loadsystemWFfile(obj)
            %load system wavefront (zernike coefficient) from file.
            [filename,~,fi]=uigetfile('*.mat','Choose Wavefront data file');
            if fi==0
                return
            end
            systemDMdata=importdata(filename);
            obj.systemDMdata=systemDMdata;
            
            
        end
        function loadsampleWFfile(obj)
            %load sample wavefront (zernike coefficient) from file.
            [filename,~,fi]=uigetfile('*.mat','Choose Wavefront data file');
            if fi==0
                return
            end
            sampleDMdata=importdata(filename);
            obj.sampleDMdata=sampleDMdata;
            
            
        end
        
        function loadZ2Cfile(obj)
            [filename,~,fi]=uigetfile('*.mat','Choose Z2C matrix file');
            if fi==0
                return
            end
            Z2C=importdata(filename);
            obj.hAOdata.Z2Cmtx=Z2C;
        end
        
        function callcameraGUI(obj)
            obj.hCameraGUI=CameraGUI(obj);
            
            
        end
        
        function usesystemWF(obj,value)
            if value==0
                obj.tousesystemWF=0;
            else
                if isempty(obj.systemDMdata)
                    msgbox('system DM data not loaded','Error')
                    obj.hGUI.Children(obj.findchild(obj.hGUI.Children,'systemabber')).Value=0; %coundn't figure
                   % out how to disable from here
                    return
                end
                
                if obj.tousesampleWF==0
                    obj.tousesystemWF=1;
                    obj.useDM=1;
                    obj.applyDMdata=obj.systemDMdata; %if only systemDMdata selected, dm data sent is just systemDMdata.
                else %if both wavefront is selected, then check if they are the same length.
                  if ~isequal(size(obj.sampleDMdata),size(obj.systemDMdata))
                     msgbox('sample and system DM data donnot have the same size','Error')
                     obj.hGUI.systemabber.Value=0;
                     return
                  else
                      obj.applyDMdata=obj.systemDMdata+obj.sampleDMdata;
                      obj.tousesampleWF=1;
                      obj.useDM=1;
                  end
                    
                end
                
                
                
            end
        
        
        end
        
        function usesampleWF(obj,value)
            if value==0
                obj.tousesampleWF=0;
            else
                if isempty(obj.sampleDMdata)
                    msgbox('sample DM data not loaded','Error')
                    obj.hGUI.Children(obj.findchild(obj.hGUI.Children,'sampleabber')).Value=0; %Couldn't figure
                    %out how to control enabl
                    return
                end
                
                if obj.tousesystemWF==0
                    obj.tousesampleWF=1;
                    obj.useDM=1;
                    obj.applyDMdata=obj.sampleDMdata; %if only systemDMdata selected, dm data sent is just systemDMdata.
                else %if both wavefront is selected, then check if they are the same length.
                  if ~isequal(size(obj.sampleDMdata),size(obj.systemDMdata))
                     msgbox('sample and system DM data donnot have the same size','Error')
                     obj.hGUI.sampleabber.Value=0;
                     return
                  else
                      obj.applyDMdata=obj.systemDMdata+obj.sampleDMdata;
                      obj.tousesampleWF=1;
                      obj.useDM=1;
                  end
                    
                end
                
                
                
            end
        end
        
        
        %static method for finding object of specific string from children
        %array in hGUI
        function indx=findchild(obj,Children,string)
            %for simplicity this only find the first object with specific
            %string, avoid using same string for different object anyways.
           nchildren=length(Children);
           for i=1:nchildren
              if strcmp(Children(i).Tag,string)
                  indx=i;
                  break
              end
           end
           
        end
        
        
        
        %Additional method for controlling pockels cell look up table
        function loadPKCLUT(obj)
            load('powerLUT.mat')
            obj.hSI.hBeams.calibrationLUT=powerLUT;
        end
        
        function savePKCLUT(obj)
            powerLUT=obj.hSI.hBeams.calibrationLUT;
            save('powerLUT.mat','powerLUT');
        end
        
    end
    
 

    
    

end
