classdef AOdata<handle
    %handle calss, only one wavefront data can be present at a time
    properties
        % datas for deformable mirror
        systemWFdata %system abberation wavefront data stored in zernike coefficient [nROIs,maxzernikeN]
        sampleWFdata %sample abberation wavefront data stored in zernike coefficient [nROIs,maxzernikeN]
        Z2Cmtx %Z2C matrix to be used when asked about DM data
        
        ROIGroup % copy of ROI group associated with this wavefront.
        wavefrontdataquality % array of number indicated wavefront data quality
        expotime % array recording the actural exposure time used when creating this wavefront.
        zscanused % whether zscan is used during wavefront measurment.
        %shrinkfraction % what shrink fraction is used in wavefront measurment.
        nROI  % number of ROI in ROIGroup.
        filename % name of tif file of this wavefront measurment.
    end
    
    %lifecycle
    methods
        function obj=AOdata(hROIGroup)
            obj.ROIGroup=copy(hROIGroup);
            obj.systemabber=[]; % creat empty field
            obj.sampleabber=[]; % creat empty field
            
            
        end
        
        function delete(~)
            %empty
        end
    end
    %User properties
    methods
%         function addWF(obj,fnSHdata,nROI,expotime,zscanused,shrinkfraction)
%             obj.expotime(nROI)=expotime;
%             obj.zscannused(nROI)=zscannused;
%             obj.shrinkfraction(nROI)=shrinkfraction;
%             obj.SHdata(nROI)=fnSHdata;
%             wfdata=obj.SHtoWF(fnSHdata);
%             obj.wavefrontdata(nROI)=wfdata;
%             obj.DMdata=obj.WFtoDM(wfdata);
%         end
%        
        %get dm command methods
        function DMdata=getsystemDMdata(obj)
            [nROIs,Nzerused]=size(obj.systemWFdata,2);
            Nzersupported=size(obj.Z2Cmtx,1);
            if Nzerused<=Nzersupported
                DMdata=[obj.systemWFdata,zeros(nROIs,Nzersupported-Nzerused)]*obj.Z2Cmtx;
            else
                ErrM=MException('AOdata: Z2Cmatrix size too small','Z2C matrix loaded support smaller number of zernike coefficient than used in system WF data');
                throw(ErrM)
            end
            
           
        end
        
        function DMdata=getsampleDMdata(obj)
            [nROIs,Nzerused]=size(obj.sampleWFdata,2);
            Nzersupported=size(obj.Z2Cmtx,1);
            if Nzerused<=Nzersupported
                DMdata=[obj.sampleWFdata,zeros(nROIs,Nzersupported-Nzerused)]*obj.Z2Cmtx;
            else
                ErrM=MException('AOdata: Z2Cmatrix size too small','Z2C matrix loaded support smaller number of zernike coefficient than used in sample WF data');
                throw(ErrM)
            end
            
           
        end
        
        function DMdata=getcombinedDMdata(obj)
            [nROIs,Nzerused]=size(obj.systemWFdata,2);
            Nzersupported=size(obj.Z2Cmtx,1);
            if Nzerused<=Nzersupported
                systemDMdata=[obj.systemWFdata,zeros(nROIs,Nzersupported-Nzerused)]*obj.Z2Cmtx;
            else
                ErrM=MException('AOdata: Z2Cmatrix size too small','Z2C matrix loaded support smaller number of zernike coefficient than used in WF data');
                throw(ErrM)
            end
            [nROIs,Nzerused]=size(obj.sampleWFdata,2);
            Nzersupported=size(obj.Z2Cmtx,1);
            if Nzerused<=Nzersupported
                sampleDMdata=[obj.sampleWFdata,zeros(nROIs,Nzersupported-Nzerused)]*obj.Z2Cmtx;
            else
                ErrM=MException('AOdata: Z2Cmatrix size too small','Z2C matrix loaded support smaller number of zernike coefficient than used in sample WF data');
                throw(ErrM)
            end
           DMdata=systemDMdata+sampleDMdata;
        end
        
        
        
        function ROIGroup=getROIGroup(obj)
            ROIGroup=copy(obj.ROIGroup);
        end
        
        function good=iscompelete(obj,usesystemWF,usesampleWF)
            % check integrity of AOdata, required before actually using them.
            good=true;
           if usesystemWF==1
               if isempty(obj.systemWFdata)
                   good=false;
                   msgbox('system wavefront data not loaded','Error')
                   return
               elseif ~size(obj.systemWFdata,1)==obj.nROI
                   good=false;
                   msgbox('system wavefront data does not match number of ROI','Error')
                   return
                   
               end
           end
           if usesampleWF==1
               if isempty(obj.sampleWFdata)
                   good=false;
                   msgbox('sample wavefront data not loaded','Error')
                   return
               elseif ~size(obj.sampleWFdata,1)==obj.nROI
                   good=false;
                   msgbox('sample wavefront data does not match number of ROI','Error')
                   return
                   
               end
           end
        end
            
    end
    
   
    
end