% Rui changes some paras
% Calibration example. Adjust the parameters of your lenslet array and use
% your SHWFS reference image.


% 
% clear all;
% close all;
% clc;
% %%
% addpath('../examples/');
% addpath('../data/');
% %%
% sh_flat=imread('ref_olympus25X_20160422.tif');
% sh_flat_bg=imread('ref_olympus25X_20160422_bg.tif');


%% parameters
shstruct = struct();
shstruct.calibration_date = datestr(now, 'yyyymmddHHMMSS');
shstruct.use_bg = 0;               % subtract background from images
shstruct.use_gausscorrection = 1; %using gauss correction for DM calibration
%shstruct.thresh_binary_img = 0.06;%0.08 % threshold for binary image coarse grid
shstruct.thresh_fine_img = 0.48;%0.55  % threshold for correlation image area detect
shstruct.npixsmall = 8; %8           % remove objects less than npixsmall
shstruct.npixsmall_corr = 9; %6        % remove objects less than npixsmall for corr-image
shstruct.strel_rad = 8;              % strel ratio (image processing in coarse grid)
shstruct.strel_rad_corr = 8;         % strel ratio (image processing in corr-image)
shstruct.coarse_grid_radius = round((500/13)/2)-1;  % radius for coarse grid
shstruct.percent = 0.8;%0.2            % threshold used in centroid fine grid
                                       % this value shoule be bigger when
                                       % more noisy of the image
                                       % scale estimated radius in the fine grid 
% define the size and sigma of gaussian filter used for psf
shstruct.psfsize_DM = 31;
shstruct.psfsize_sample = 31;
shstruct.psfsigma_sample = 1.5;%change from 1 to 1.5, want the osf bigger
                              % psfsigma value is associate with lensarray
                              % focal lenght
shstruct.psfsigma_DM = 1.5;%change from 1.5 to 1,
% this caused the fine grid to be smaller than the course grid
shstruct.multiply_est_radius = 0.9; %1/sqrt(2)
shstruct.centroid = @centroid;     % centroid algorithm
% lenslet documentation
shstruct.totlenses = 170;
shstruct.flength = 41.7e-3;
shstruct.pitch = 500e-6;
% camera documentation
shstruct.camera_pixsize = 13*1e-6;
shstruct.maxinteger = 2^16 - 1;       % maximum integer value for the image
% params Dai modal wavefront estimation
shstruct.sa_radius_m = shstruct.pitch/2;
shstruct.dai_n_zernike = 10;%5  % max radial order of the Zernike polynomials


%% pre-process of sh_flat
% replace your reference SHWFS images here
% DM_calibration_shdata_ref(389:603,525:548)=525;
sh_flat =  DM_calibration_shdata_ref;
sh_flat_bg = DM_calibration_shdata_bg;
% subtract background
sh_flat = sh_flat-int16(sh_flat_bg); 
% aperture template cut 
imgtemp = int16(zeros(1024,1024));
xcenter = 495;
ycenter = 491;
imgrad = 252;%real aperture size 272-286; beam size 286
for x = 1:1024
    for y = 1:1024
        if sqrt((x-xcenter)^2+(y-ycenter)^2)<=imgrad
            imgtemp(y,x)=1;
        else
            imgtemp(y,x)=0;
        end
    end
end
sh_flat = sh_flat.*imgtemp;

%Gauss correction
gautemp = fspecial('gaussian',1024,imgrad);% the radius of the gauss temp can be smaller 
gautemp =gautemp.^2; % 3 power is better than 2 power
gautemp = (gautemp-min(min(gautemp)))./(max(max(gautemp))-min(min(gautemp)));
if shstruct.use_gausscorrection
    sh_flat = int16(double(sh_flat)./gautemp);
else
    sh_flat;
end


% generate threshould for course grid cal
shstruct.thresh_course_img  = graythresh(sh_flat);%%%this is crucial for grid generation

%% run the calibration
%shwfs_calibrate;

mkdir(sprintf('calibration-%s', shstruct.calibration_date));
%% background
% sfigure(1);
% imshow(sh_flat_bg);
% title('background');
% drawnow();
% pause(.1);

%% flat mirror reference
% Rui<this section do nothing>
figure;
imshow(sh_flat);
imagesc(sh_flat);
title('reference image with gauss correction');
iimin = min(sh_flat(:));
iimax = max(sh_flat(:));
fprintf('[min, max] = [%.2f %.2f]\n', iimin, iimax);

shstruct.sh_flat = sh_flat;
shstruct.sh_flat_bg = sh_flat_bg;
shstruct.aperture_temp = imgtemp;

if exist('scflat', 'var')
    shstruct.scflat = scflat;
end

% filename_blank = ...
%     sprintf('calibration-%s/shstruct.blank.mat', ...
%     shstruct.calibration_date);
% ask_confirm('continue?');
% 
% save(filename_blank, 'shstruct');
% fprintf('saved %s\n', filename_blank);

%% coarse grid with image processing
%load(filename_blank);

%shstruct.thresh_course_img = 0.58;
shstruct = shwfs_make_coarse_grid(shstruct);
filename_coarse = ...
    sprintf('calibration-%s/shstruct.coarse.mat', ...
    shstruct.calibration_date);
%ask_confirm('continue?');

save(filename_coarse, 'shstruct');
fprintf('saved %s\n', filename_coarse);
%% finer grid
load(filename_coarse);
shstruct = shwfs_make_fine_grid(shstruct);
filename_fine = ...
    sprintf('calibration-%s/shstruct.fine.mat', ...
    shstruct.calibration_date);
save(filename_fine, 'shstruct');
fprintf('saved %s\n', filename_fine);

%% make dai
load(filename_fine);
shstruct = shwfs_make_dai(shstruct);
filename_dai = ...
    sprintf('calibration-%s/shstruct.dai.mat', ...
    shstruct.calibration_date);
%ask_confirm('continue?');

save(filename_dai, 'shstruct');
fprintf('saved %s\n', filename_fine);

%% make DM calibration slope matrix 
% using 60/51 orders
zernikeorder=60;
nspots = shstruct.nspots;
ref_cernter = shstruct.ord_centres;
h = waitbar(0,'DM calibration calculation...');
for i=1:zernikeorder
img = int16(DM_calibration_shdata(:,:,i));
img = img - int16(shstruct.sh_flat_bg);
aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
slope(:,i) = reshape(aberrated,nspots*2,1) - reshape(ref_cernter,nspots*2,1);
waitbar(i/zernikeorder);
end
close(h);
Geninv = pinv(slope);
shstruct.DMcalibration_slope = slope;
shstruct.Geninv = Geninv;           

filename_fine = ...
    sprintf('calibration-%s/shstruct.final.60.mat', ...
    shstruct.calibration_date);
save(filename_fine, 'shstruct');

%% DM calibration show based on ideal zernike poynim
figure;
lambda=700e-9;
h = waitbar(0,'Drawing...');
for i=1:20
    %draw ideal zernike pattern(zernike index to zernike pattern)
    
    zrad=zeros(65,1);
    %zrad(1:45,1)=index(:,1);
    zrad(i,1)=1;  
    
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    se = flen*(shstruct.dai_E1)*zrad;
    z = ((2*pi)/(lambda))*zrad;

    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);    
    zcs = [0; z];
    sel = 1:size(z, 1);
    
    subplot(5,10,2*i-1);
    zernike_imagesc(zstruct, zcs);
    
    %draw DM zernike pattern(image to zernike pattern) 
    img=int16(DM_calibration_shdata(:,:,i));
     
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
    sdeltas = aberrated - shstruct.ord_centres;
    t = ps*sdeltas;
    s = t(:);
    zrad = (1/flen)*(shstruct.dai_pE1*s);
    se = flen*(shstruct.dai_E1)*zrad;
    er = s - se;
    zhatrad = ((2*pi)/(lambda))*zrad;
  
    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);
    
    zcs = [0; zhatrad];
    sel = 1:size(zhatrad, 1);
    
    subplot(5,10,2*i);
    zernike_imagesc(zstruct, zcs);
    waitbar(i/20);
end
close(h);
%%
figure;
h = waitbar(0,'Drawing...');
for m = 1:(40-20)
    %draw ideal zernike pattern(zernike index to zernike pattern)
    i = m+20;
    zrad=zeros(65,1);
    %zrad(1:45,1)=index(:,1);
    zrad(i,1)=1;  
    
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    se = flen*(shstruct.dai_E1)*zrad;
    z = ((2*pi)/(lambda))*zrad;

    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);    
    zcs = [0; z];
    sel = 1:size(z, 1);
    
    subplot(5,10,2*m-1);
    zernike_imagesc(zstruct, zcs);
    
    %draw DM zernike pattern(image to zernike pattern) 
    img=int16(DM_calibration_shdata(:,:,i));
     
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
    sdeltas = aberrated - shstruct.ord_centres;
    t = ps*sdeltas;
    s = t(:);
    zrad = (1/flen)*(shstruct.dai_pE1*s);
    se = flen*(shstruct.dai_E1)*zrad;
    er = s - se;
    zhatrad = ((2*pi)/(lambda))*zrad;
  
    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);
    
    zcs = [0; zhatrad];
    sel = 1:size(zhatrad, 1);
    
    subplot(5,10,2*m);
    zernike_imagesc(zstruct, zcs);
    waitbar(m/(40-20));
end
close(h);
%%
figure;
h = waitbar(0,'Drawing...');
for m = 1:(60-40)
    %draw ideal zernike pattern(zernike index to zernike pattern)
    i = m+40;
    zrad=zeros(65,1);
    %zrad(1:45,1)=index(:,1);
    zrad(i,1)=1;  
    
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    se = flen*(shstruct.dai_E1)*zrad;
    z = ((2*pi)/(lambda))*zrad;

    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);    
    zcs = [0; z];
    sel = 1:size(z, 1);
    
    subplot(5,10,2*m-1);
    zernike_imagesc(zstruct, zcs);
    
    %draw DM zernike pattern(image to zernike pattern) 
    img=int16(DM_calibration_shdata(:,:,i));
     
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
    sdeltas = aberrated - shstruct.ord_centres;
    t = ps*sdeltas;
    s = t(:);
    zrad = (1/flen)*(shstruct.dai_pE1*s);
    se = flen*(shstruct.dai_E1)*zrad;
    er = s - se;
    zhatrad = ((2*pi)/(lambda))*zrad;
  
    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);
    
    zcs = [0; zhatrad];
    sel = 1:size(zhatrad, 1);
    
    subplot(5,10,2*m);
    zernike_imagesc(zstruct, zcs);
    waitbar(m/(60-40));
end
close(h);

%draw as indivudual figure


lambda=725e-9;
%%
for i=1:60;
    %draw ideal zernike pattern(zernike index to zernike pattern)
    zrad=zeros(65,1);
    %zrad(1:45,1)=index(:,1);
    zrad(i,1)=1;  
    
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    se = flen*(shstruct.dai_E1)*zrad;
    z = ((2*pi)/(lambda))*zrad;

    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 4000);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);    
    zcs = [0; z];
    sel = 1:size(z, 1);
    
    
    figure,
    zernike_imagesc(zstruct, zcs);
    axis image;
    II=gcf;
    saveas(II,sprintf('%s%d%s','zernike',i,'.fig')); 
    
    
    
    %draw DM zernike pattern(image to zernike pattern) 
    img=int16(DM_calibration_shdata(:,:,i));
     
    flen = shstruct.flength;
    ps = shstruct.camera_pixsize;
    aberrated = shwfs_get_centres_nodraw(img, shstruct, shstruct.centroid);
    sdeltas = aberrated - shstruct.ord_centres;
    t = ps*sdeltas;
    s = t(:);
    zrad = (1/flen)*(shstruct.dai_pE1*s);
    se = flen*(shstruct.dai_E1)*zrad;
    er = s - se;
    zhatrad = ((2*pi)/(lambda))*zrad;
  
    zstruct = shstruct.dai_zstruct;
    dd = linspace(-1, 1, 80);
    [xx, yy] =  meshgrid(dd, dd);
    zstruct = zernike_cache(zstruct, xx, yy);
    
    zcs = [0; zhatrad];
    sel = 1:size(zhatrad, 1);
    
    figure;
    zernike_imagesc(zstruct, zcs);
        axis image;
      II=gcf;
    saveas(II,sprintf('%s%d%s','DM',i,'.fig')); 
end

