% SHWFS_MAKE_FINE_GRID.
%   [SHSTRUCT] = SHWFS_MAKE_FINE_GRID(SHSTRUCT). Make a fine grid to
%   compute the centroids using the image processing toolbox.
%
% Author: Jacopo Antonello, <jack@antonello.org>
% Technische Universiteit Delft

function [shstruct] = shwfs_make_fine_grid(shstruct)

sh_flat = shstruct.sh_flat;

%% get centres with coarse grid
nspots = shstruct.nspots;
centres = zeros(nspots, 2);
img = shstruct.sh_flat;

thresh_fine_img = shstruct.thresh_fine_img;
npixsmall_corr = shstruct.npixsmall_corr;
strel_rad_corr = shstruct.strel_rad_corr;
psf_size = shstruct.psfsize_DM;
psf_sigma = shstruct.psfsigma_DM;

img_corr = corsscorrelation(psf_size,psf_sigma,img);

for i=1:nspots
    cc = shstruct.squaregrid(i, :);% get squaregrid from coarse grid

    subimage = img(cc(3):cc(4), cc(1):cc(2));
    subimage_corr = img_corr(cc(3):cc(4), cc(1):cc(2));
    
    iimax = max(max(subimage_corr));
    iimin = min(min(subimage_corr));
    
    level = (iimax - iimin)*thresh_fine_img; 
    subimage_corr = subimage_corr - iimin;
    subimage_corr_bw = im2bw(subimage_corr, level);
    
    %remove small objects
    subimage_corr_bw = bwareaopen(subimage_corr_bw, npixsmall_corr);
    % remove edges
    se = strel('disk',strel_rad_corr);
    subimage_corr_bw = imclose(subimage_corr_bw, se);
 
    if sum(sum(subimage_corr_bw))==0
        subimage_corr_bw(round(size(subimage_corr_bw,1)/2),...
            round(size(subimage_corr_bw,1)/2)) = 1;
    end
        
    
    subimage_thr = subimage.*int16(subimage_corr_bw);
    
    % level = (iimax - iimin)*shstruct.percent + iimin;   
     dd = shstruct.centroid(subimage_thr);
    % centroid returns in column major order
     centres(i, :) = [cc(1)+dd(2)-1, cc(3)+dd(1)-1];   

%     old centroid    
%     iimin = min(min(subimage));
%     iimax = max(max(subimage));
%     level = (iimax - iimin)*shstruct.percent + iimin;    
%     dd = shstruct.centroid(subimage, level);
%     centres(i, :) = [cc(1)+dd(2)-1, cc(3)+dd(1)-1];
    
  
    
    
    
end
%%
shstruct.centres = centres;
fprintf('any non finite? %d\n', any(any(isfinite(centres) == 0)));
fprintf('min(sh_flat(:)) %f\n', min(sh_flat(:)));
fprintf('max(sh_flat(:)) %f\n', max(sh_flat(:)));
%Rui block
%fprintf('min(exp(sh_flat(:))) %f\n', min(exp(sh_flat(:))));
%fprintf('max(exp(sh_flat(:))) %f\n', max(exp(sh_flat(:))));
%% guess central spot
norms = zeros(nspots, nspots);
for i=1:nspots
    for j=1:nspots
        norms(i, j) = norm(centres(i, :) - centres(j, :));
        %the same as: max(svd(centres(i, :) - centres(j, :)))
    end
end
lnorms = sum(norms, 2);
[~, icentralspot] = min(lnorms);
% estimate radius
mins = zeros(nspots, 1);
for i=1:nspots
    [~, sel] = sort(norms(i, :));
    mins(i) = norms(i, sel(2));
end
radius = mean(mins)/2*shstruct.multiply_est_radius;

figure;
plot(mins, 'o');
xlabel('spot number');
ylabel('radius');
drawnow()
pause(.1);

% estimate pupil radius
shstruct.est_pupil_radius_m = ...
    max(norms(icentralspot, :))*...
    shstruct.camera_pixsize;

shstruct.squaregrid_coarse = shstruct.squaregrid;
shstruct.squaregrid_radius = radius;% radius is not round number

for i=1:nspots
    c = centres(i, :); % centres is not round number
    minx = c(1) - radius;
    maxx = c(1) + radius;
    miny = c(2) - radius;
    maxy = c(2) + radius;
    shstruct.squaregrid(i, :) = round([minx, maxx, miny, maxy]);%caused the unregular of the grid size
end
%% draw fine grid
grid = shstruct.squaregrid;

figure;
imshow(sh_flat);
hold on;
spotxy = centres(icentralspot, :);
plot(spotxy(1), spotxy(2), 'yo');

title('fine grid');
for i=1:nspots
    cc = grid(i, :);
    rectangle('Position', ...
        [cc(1), cc(3), cc(2)-cc(1)+1, cc(4)-cc(3)+1], ...
        'LineWidth', 1, 'EdgeColor', 'b');
    plot(centres(i, 1), centres(i, 2), 'xr');
    text(cc(1), cc(3), ...
        sprintf('%d', i), 'Color', 'g');
end
hold off;
%% unused, code for zonal is outdated
shstruct.enumeration = 1:shstruct.nspots;
shstruct.ord_centres = shstruct.centres(shstruct.enumeration, :);
shstruct.ord_sqgrid = zeros(size(shstruct.ord_centres, 1), 4);
shstruct.ord_sqgrid = shstruct.squaregrid(shstruct.enumeration, :);


end
